/*
Title: MINEFIELD 6D
Author: Igor Galochkin, iseeall@narod.ru
Version: 1.2.1
Project started: 17 April 2006
Last update: 29 June 2006
*/

#include "minefield_6d.h"
#include "resource.h"

//---------------------------------------------------------------------------
void placeBombs()//Randomly places bombs
{
	vector<int> bomb_pos (MAX_DIMS);	//random coords of next bomb
	vector<int> neighbor_pos (MAX_DIMS);	// coords of its currently checked neighbor
	
	for (int num_placed_bombs = 0; num_placed_bombs < bombs;)
	{
		for (int j = 0; j != MAX_DIMS; ++j)		//give random coordinates to the bomb
			bomb_pos [j] = randomInt (field_size [j]);
			
		int n = toIndex (bomb_pos);
		assert (n >= 0 && n < field_volume);
		
		if (neighbors [n] != BOMB) // place bomb if this tile is not already a bomb
		{
			neighbors [n] = BOMB;	
			++num_placed_bombs;
		
		// increase value of all neighbors by	1	

			for (int j = 0; j != MAX_DIMS; ++j)
				neighbor_pos [j] = max (0, bomb_pos [j] - 1); 
			
			while (true)
			{ 
				int n1 = toIndex (neighbor_pos);
				if (neighbors [n1] != BOMB)
					++neighbors[n1];
				
				//cycle thru all neighbors
				++neighbor_pos[0]; 
				for (int j = 0; j != MAX_DIMS - 1; ++j)
					if (neighbor_pos[j] == min (bomb_pos [j] + 2 , field_size [j]))
					{
						neighbor_pos [j] = max (0, bomb_pos [j] - 1); 
						++ neighbor_pos [j + 1];
					}
				if (neighbor_pos [MAX_DIMS - 1] == min (bomb_pos [MAX_DIMS - 1] + 2, field_size [MAX_DIMS - 1]))
					break; //break loop at end point
			}
		}
	}//all bombs placed
}//end place bombs

//---------------------------------------------------------------------------
bool revealEmpty()	// open all empty tiles and their neighbors. 
// This option is best to run right after the field is created for field with >2 dims
{
	assert (not_opened_cells == field_volume);
	vector<int> tile_pos (MAX_DIMS);
	vector<int> neighbor_pos (MAX_DIMS);
	for (int n = 0; ; ++n)
	{
		if (neighbors [n] == 0)
		{					
			for (int j = 0; j != MAX_DIMS; ++j)
				neighbor_pos [j] = max (0, tile_pos [j] - 1);
				
			while (true)
			{ 
				int n1 = toIndex (neighbor_pos);
				if (neighbors [n1] != BOMB &&
						tile_state [n1] == CLOSED)
				{
					tile_state[n1] = OPENED;
					--not_opened_cells;
				}
				
				//cycle thru all neighbors
				++neighbor_pos[0]; 
				for (int j = 0; j != MAX_DIMS - 1; ++j)
					if (neighbor_pos[j] == min (tile_pos [j] + 2 , field_size [j]))
					{
						neighbor_pos [j] = max (0, tile_pos [j] - 1); 
						++ neighbor_pos [j + 1];
					}
				if (neighbor_pos [MAX_DIMS - 1] == min (tile_pos [MAX_DIMS - 1] + 2, field_size [MAX_DIMS - 1]))
					break; //break loop at end point
			}
		}
		if (!nextFieldElement (tile_pos))
			break;
	}
	return (!victoryAchieved()); // if revealing empty cells lead to instant victory, 
																//return false indicating field creating failure
}

//---------------------------------------------------------------------------
void createField()
{
	if (dims <= 6)
	{
	//calculate width and height of tables on screen
		size_in_pixels[0] = TILE_SIZE * field_size [0] + SPACE01; 
		size_in_pixels[1] = TILE_SIZE * field_size [1] + SPACE01; //width of 2-dim table
		size_in_pixels[2] = size_in_pixels [0] * field_size [2] + SPACE23;
		size_in_pixels[3] = size_in_pixels [1] * field_size [3] + SPACE23; //width of 4-dim table
		size_in_pixels[4] = size_in_pixels [2] * field_size [4] - SPACE23 - SPACE01;
		size_in_pixels[5] = size_in_pixels [3] * field_size [5] - SPACE23 - SPACE01; //width of 6-dim table
		
		// Resize the window to be just big enough to show all the tiles
		screen_width = size_in_pixels[4] + BORDER * 2;
		screen_height = size_in_pixels[5] + BORDER * 2;
	}
	else // more than 6 dims: twice smaller tiles
	{
		//calculate width and height of tables on screen
		size_in_pixels[0] = TILE_SIZE_2 * field_size [0] + SPACE01_2; 
		size_in_pixels[1] = TILE_SIZE_2 * field_size [1] + SPACE01_2; //width of 2-dim table
		size_in_pixels[2] = size_in_pixels [0] * field_size [2] + SPACE23_2;
		size_in_pixels[3] = size_in_pixels [1] * field_size [3] + SPACE23_2; //width of 4-dim table
		size_in_pixels[4] = size_in_pixels [2] * field_size [4] + SPACE45_2;
		size_in_pixels[5] = size_in_pixels [3] * field_size [5] + SPACE45_2;// width of 6-dim table
		size_in_pixels[6] = size_in_pixels [4] * field_size [6] + SPACE67_2;
		size_in_pixels[7] = size_in_pixels [5] * field_size [7] + SPACE67_2; // width of 8-dim table
		size_in_pixels[8] = size_in_pixels [6] * field_size [8] 
								- SPACE89_2 - SPACE67_2 - SPACE45_2 - SPACE23_2 - SPACE01_2;
		size_in_pixels[9] = size_in_pixels [7] * field_size [9] 
								- SPACE89_2 - SPACE67_2 - SPACE45_2 - SPACE23_2 - SPACE01_2; 
																		//width of 10-dim table
		
		// Resize the window to be just big enough to show all the tiles
		screen_width = size_in_pixels[8] + BORDER_2 * 2;
		screen_height = size_in_pixels[9] + BORDER_2 * 2;
	}
	SetWindowPos(hwnd, NULL, 0, 0, 
							screen_width + 2 * GetSystemMetrics (SM_CXFIXEDFRAME), 
							screen_height + GetSystemMetrics (SM_CYCAPTION) + 
								GetSystemMetrics (SM_CYMENU) + 2 * GetSystemMetrics (SM_CYFIXEDFRAME), 
							SWP_NOMOVE | SWP_NOZORDER | SWP_NOACTIVATE);
	
	//setting multipliers (digits) to use it when calculating address of array elements
	multiplier [0] = 1; 
	for(int n = 1; n != MAX_DIMS; ++n)
		multiplier [n] = multiplier [n - 1] * field_size [n - 1];

 	
	field_volume = field_size [MAX_DIMS - 1] * multiplier [MAX_DIMS - 1]; //total number of cells on the field
	
	vector<int> temp (field_volume, 0); 
	pixel_pos_x = pixel_pos_y = tile_state = neighbors = temp; // get vectors of required size filled with zeros
	
	not_opened_cells = field_volume; 
	// setting up positions of each tile on the screen (in pixels). 
	// It is better to do it in advance so as not 
	// to calculate again at each repaint
	int p2(0), p3(0), p4(0), p5(0), p6(0), p7(0), p8(0), p9(0); 
	vector<int> i (MAX_DIMS);	
	if (dims <= 6)
	{	
		for (int n = 0; ; ++n)
		{
			p5 = i[5] * size_in_pixels[3]; 
			p4 = i[4] * size_in_pixels[2];
			p3 = p5 + i[3] * size_in_pixels[1]; 
			p2 = p4 + i[2] * size_in_pixels[0];
			
			pixel_pos_y [n] = p3 + TILE_SIZE * i[1] + BORDER;
			pixel_pos_x [n] = p2 + TILE_SIZE * i[0] + BORDER;
			
			if (!nextFieldElement (i))
				break;
		}
	}
	else	//dims number from 7 to 10
	{
		for (int n = 0; ; ++n)
		{
			p9 = i[9] * size_in_pixels[7];
			p8 = i[8] * size_in_pixels[6];
			p7 = p9 + i[7] * size_in_pixels[5];
			p6 = p8 + i[6] * size_in_pixels[4];			
			p5 = p7 + i[5] * size_in_pixels[3]; 
			p4 = p6 + i[4] * size_in_pixels[2];
			p3 = p5 + i[3] * size_in_pixels[1]; 
			p2 = p4 + i[2] * size_in_pixels[0];
			
			pixel_pos_y [n] = p3 + TILE_SIZE_2 * i[1] + BORDER_2;
			pixel_pos_x [n] = p2 + TILE_SIZE_2 * i[0] + BORDER_2;
			
			if (!nextFieldElement (i))
				break;
		}
	}
}

//-------------------------------------------------------------------------------
void updateCoordsOfTileUnderCursor (int mouse_x, int mouse_y)
{
	// function not only checks over which tile the cursor is, but also checks if cursor hits border areas between tables
	// in that case it returns CURSOR_NOT_ON_ANY_TILE

	cursor_tile_coord[0] = CURSOR_NOT_ON_ANY_TILE;

	if (dims <=6)
	{	
		mouse_y -= BORDER;
		mouse_x -= BORDER;
		if (mouse_y > 0 && mouse_y < size_in_pixels[5] &&
				mouse_x > 0 && mouse_x < size_in_pixels[4])
		{
			cursor_tile_coord[5] = mouse_y / size_in_pixels[3];
			cursor_tile_coord[4] = mouse_x / size_in_pixels[2];
			mouse_y -= cursor_tile_coord[5] * size_in_pixels[3];
			mouse_x -= cursor_tile_coord[4] * size_in_pixels[2];
			
			if (mouse_y < size_in_pixels[3] - SPACE23 &&
					mouse_x < size_in_pixels[2] - SPACE23)
			{
				cursor_tile_coord[3] = mouse_y / size_in_pixels[1];
				cursor_tile_coord[2] = mouse_x / size_in_pixels[0];
				mouse_y -= cursor_tile_coord[3] * size_in_pixels[1];
				mouse_x -= cursor_tile_coord[2] * size_in_pixels[0];
				
				if (mouse_y < size_in_pixels[1] - SPACE01 &&
						mouse_x < size_in_pixels[0] - SPACE01)
				{
					cursor_tile_coord[1] = mouse_y / TILE_SIZE;
					cursor_tile_coord[0] = mouse_x / TILE_SIZE;
				}
			}
		}	
	}
	else	// iron man mode, up to 10 dims, tiles are twice smaller, spaces btw tables 5 times smaller
	{
		mouse_y -= BORDER_2;
		mouse_x -= BORDER_2;
		if (mouse_y > 0 && mouse_y < size_in_pixels[9] &&
				mouse_x > 0 && mouse_x < size_in_pixels[8])
		{			
			cursor_tile_coord[9] = mouse_y / size_in_pixels[7];
			cursor_tile_coord[8] = mouse_x / size_in_pixels[6];
			mouse_y -= cursor_tile_coord[9] * size_in_pixels[7];
			mouse_x -= cursor_tile_coord[8] * size_in_pixels[6];
			
			if (mouse_y < size_in_pixels[7] - SPACE67_2 &&
					mouse_x < size_in_pixels[6] - SPACE67_2)
			{
				cursor_tile_coord[7] = mouse_y / size_in_pixels[5];
				cursor_tile_coord[6] = mouse_x / size_in_pixels[4];
				mouse_y -= cursor_tile_coord[7] * size_in_pixels[5];
				mouse_x -= cursor_tile_coord[6] * size_in_pixels[4];
				
				if (mouse_y < size_in_pixels[5] - SPACE45_2 &&
						mouse_x < size_in_pixels[4] - SPACE45_2)
				{
					cursor_tile_coord[5] = mouse_y / size_in_pixels[3];
					cursor_tile_coord[4] = mouse_x / size_in_pixels[2];
					mouse_y -= cursor_tile_coord[5] * size_in_pixels[3];
					mouse_x -= cursor_tile_coord[4] * size_in_pixels[2];
					
					if (mouse_y < size_in_pixels[3] - SPACE23_2 &&
							mouse_x < size_in_pixels[2] - SPACE23_2)
					{
						cursor_tile_coord[3] = mouse_y / size_in_pixels[1];
						cursor_tile_coord[2] = mouse_x / size_in_pixels[0];
						mouse_y -= cursor_tile_coord[3] * size_in_pixels[1];
						mouse_x -= cursor_tile_coord[2] * size_in_pixels[0];
						
						if (mouse_y < size_in_pixels[1] - SPACE01_2 &&
								mouse_x < size_in_pixels[0] - SPACE01_2)
						{
							cursor_tile_coord[1] = mouse_y / TILE_SIZE_2;
							cursor_tile_coord[0] = mouse_x / TILE_SIZE_2;
						}
					}
				}
			}
		}
	}
	cursor_tile_index = toIndex (cursor_tile_coord);
}


//---------------------------------------------------------------------------
void updateHighlight ()
{
	for (int i = 0; i != field_volume; ++i)
	{
		if (tile_state[i] == HIGHLIGHTED)
			tile_state[i] = CLOSED;
		if (tile_state[i] == HIGHLIGHTED_AND_FLAGGED)
			tile_state[i] = FLAGGED;
	}	
	if (neighbors [cursor_tile_index] != BOMB &&
		tile_state [cursor_tile_index] == OPENED)
	{
	
		vector<int> neighbor_pos (MAX_DIMS);
		for (int j = 0; j != MAX_DIMS; ++j)
					neighbor_pos [j] = max (0, cursor_tile_coord [j] - 1);
					
		while (true)
		{ 
			int n1 = toIndex (neighbor_pos);
			if (tile_state [n1] == CLOSED)
				tile_state[n1] = HIGHLIGHTED;
			if (tile_state [n1] == FLAGGED)
				tile_state[n1] = HIGHLIGHTED_AND_FLAGGED;
			
			//cycle thru all neighbors
			++neighbor_pos[0]; 
			for (int j = 0; j != MAX_DIMS - 1; ++j)
				if (neighbor_pos[j] == min (cursor_tile_coord [j] + 2 , field_size [j]))
				{
					neighbor_pos [j] = max (0, cursor_tile_coord [j] - 1); 
					++ neighbor_pos [j + 1];
				}
			if (neighbor_pos [MAX_DIMS - 1] == min (cursor_tile_coord [MAX_DIMS - 1] + 2, field_size [MAX_DIMS - 1]))
				break; //break loop at end point
		}	
	}
}

//---------------------------------------------------------------------------
// clears the field and put the first shape on the screen
void startGame (int dims1 = 2, int bombs_nr = 20, 
				int s0 = 20, int s1 = 20, int s2 = 1, int s3 = 1, int s4 = 1, int s5 = 1, 
				int s6 = 1, int s7 = 1, int s8 = 1, int s9 = 1) 
{
	field_size[0] = s0;
	field_size[1] = s1;
	field_size[2] = s2;
	field_size[3] = s3;
	field_size[4] = s4;
	field_size[5] = s5;
	field_size[6] = s6;
	field_size[7] = s7;
	field_size[8] = s8;
	field_size[9] = s9;
	dims = dims1;
	bombs = bombs_nr;
	
	createField();	
	placeBombs (); 
	if (!revealEmpty())
	{
		do
		{
			++bombs;
			createField();
			placeBombs (); 
		}
		while (!revealEmpty());
		repaint();
		//message ("Bombs numer was adjusted to %d", bombs);
	}	
	else repaint ();
}

//------------------------------------------------------------------
void restartGame ()
{
	startGame (dims, bombs, 
				field_size[0], field_size[1], field_size[2], field_size[3], field_size[4], field_size[5], 
				field_size[6], field_size[7], field_size[8], field_size[9]);
}

//------------------------------------------------------------------
void gameEnd()
{  
	// Cleanup the device context and bitmaps
	DeleteObject (hdc);
	for (vector<HBITMAP>::size_type i = 0; i != loaded_images.size(); ++i)
		DeleteObject (loaded_images[i]);
	DestroyMenu (hmenu);
}

//----------------------------------------------------------------------------

void render ()	
{
	//use either large tiles for <6 dims or smaller ones for >6 dims
	int tile_size = (dims <= 6 ? TILE_SIZE : TILE_SIZE_2);
	HBITMAP image = (dims <= 6 ? image_tiles : image_tiles_2);
	int closed_pic_x = (dims <= 6 ? 80 : 40);
	int flag_pic_x = (dims <= 6 ? 112 : 56);
	int highl_pic_x = (dims <= 6 ? 96 : 48);
	int highfl_pic_x = (dims <= 6 ? 128 : 64);
	int expl_pic_x = (dims <= 6 ? 144 : 72);
	int spec_pic_y = (dims <= 6 ? 32 : 16);
	
	for (int i = 0; i != field_volume; ++i)
		switch (tile_state [i])
		{
			case CLOSED:			
				drawImagePart (image, pixel_pos_x[i], pixel_pos_y[i], 
								closed_pic_x, spec_pic_y, tile_size, tile_size); 
			break;
			
			case FLAGGED:			
				drawImagePart (image, pixel_pos_x[i], pixel_pos_y[i], 
								flag_pic_x, spec_pic_y, tile_size, tile_size); 
			break;
			
			case HIGHLIGHTED:	
				drawImagePart (image, pixel_pos_x[i], pixel_pos_y[i], 
								highl_pic_x, spec_pic_y, tile_size, tile_size); 
			break;
			
			case HIGHLIGHTED_AND_FLAGGED: 
				drawImagePart (image, pixel_pos_x[i], pixel_pos_y[i], 
								highfl_pic_x, spec_pic_y, tile_size, tile_size); 
			break;
			
			case EXPLOSION:		
				drawImagePart (image, pixel_pos_x[i], pixel_pos_y[i], 
								expl_pic_x, spec_pic_y, tile_size, tile_size); 
			break;
			
			case OPENED:
				int sx (0), sy (0);	// position of needed image part in the combined tile image
				if (neighbors [i] >= 20)	
				{// if there are more than 20 boms near the tile, just Draw 20+ picture instead of exact number
					sx = 0;
					sy = 2 * tile_size;
				}
				else 
				{
					sx = (neighbors[i] % 10)	* tile_size;
					sy = (neighbors[i] / 10)	* tile_size;
				}
				drawImagePart (image, pixel_pos_x[i], pixel_pos_y[i], sx, sy, tile_size, tile_size); 
			break;
		}
}

//------------------------------------------------------------------------------------

BOOL CALLBACK AboutDlgProc (HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam)
{
	switch (msg)
	{
		case WM_INITDIALOG:
			return TRUE;
		case WM_COMMAND:
			switch (LOWORD (wParam))
			{
				case IDOK:
				case IDCANCEL:
					EndDialog (hDlg, FALSE);
					return TRUE;
			}
		break;
	}
	return FALSE;
}

//------------------------------------------------------------------------------------
LRESULT CALLBACK WndProc (HWND hWindow, UINT msg, WPARAM wParam, LPARAM lParam)
{
	switch (msg)
	{
		case WM_CREATE:
			startGame (2, 40, 20, 20); 
			image_tiles = loadImage ("TILES");
			image_tiles_2 = loadImage ("TILES_2");
		return 0;

		case WM_PAINT:
			PAINTSTRUCT ps;
			hdc = BeginPaint(hWindow, &ps);
			render ();
			EndPaint(hWindow, &ps);
			return 0;
		
		case WM_LBUTTONDOWN:	
		{			
			int mouse_x = LOWORD (lParam);
			int mouse_y = HIWORD (lParam);	
			updateCoordsOfTileUnderCursor (mouse_x, mouse_y);			
			if (cursor_tile_index != CURSOR_NOT_ON_ANY_TILE)
				if (tile_state[cursor_tile_index] == CLOSED
					//|| tile_state[cursor_tile_index] == FLAGGED 
					// design change 29.06.2006:
					// if bomb is flagged, then you can't open it with a click
					)
				{
					if (neighbors [cursor_tile_index] == BOMB)
					{
						for (int j = 0; j != field_volume; ++j)
							if (neighbors[j] == BOMB)
								tile_state[j] = EXPLOSION;
						repaint ();	
						message ("You lost...");
						restartGame ();
					}
					else
					{
						tile_state[cursor_tile_index] = OPENED;	
						repaint ();	
						--not_opened_cells;	
						if (victoryAchieved())
						{
							message ("YOU WIN!");
							restartGame ();
						}
					}										
				}
		}
		return 0;

		case WM_RBUTTONDOWN:	
		{			
			int mouse_x = LOWORD (lParam);
			int mouse_y = HIWORD (lParam);
			updateCoordsOfTileUnderCursor (mouse_x, mouse_y);
			if (cursor_tile_index != CURSOR_NOT_ON_ANY_TILE)
			{
				if (tile_state[cursor_tile_index] == CLOSED)
						tile_state[cursor_tile_index] = FLAGGED;
				else if (tile_state[cursor_tile_index] == FLAGGED)
						tile_state[cursor_tile_index] = CLOSED;
				repaint ();
			}
		}
		return 0;
		
		case WM_MOUSEMOVE:
			if (highlight_possible_bombs)
			{
				int mouse_x = LOWORD (lParam);
				int mouse_y = HIWORD (lParam);
				updateCoordsOfTileUnderCursor (mouse_x, mouse_y);
				
				// repaint only if cursor moved to another tile
				static int last_repaint_cursor_tile_index (0); 				
				if (last_repaint_cursor_tile_index != cursor_tile_index)
				{
					last_repaint_cursor_tile_index = cursor_tile_index;
					updateHighlight ();
					repaint ();
				}
			}
		return 0;
		
		case WM_COMMAND:	
			switch (LOWORD (wParam))
			{
				case ID_2D_EASY:	startGame (2, 40, 20, 20);	break;
				case ID_2D_NORMAL:	startGame (2, 200, 40, 30);	break;
				case ID_2D_HARD:	startGame (2, 600, 60, 40);	break;
				case ID_3D_EASY:	startGame (3, 30, 8, 20, 4);	break;
				case ID_3D_NORMAL:	startGame (3, 120, 9, 30, 5);	break;
				case ID_3D_HARD:	startGame (3, 400, 10, 40, 6);	break;
				case ID_4D_EASY:	startGame (3, 10, 8, 5, 4, 4);		break;
				case ID_4D_NORMAL:	startGame (4, 40, 9, 6, 5, 5);		break;
				case ID_4D_HARD:	startGame (4, 150, 10, 6, 6, 6);	break;
				case ID_5D_EASY:	startGame (5, 6, 4, 4, 3, 5, 3);	break;
				case ID_5D_NORMAL:	startGame (5, 20, 4, 4, 4, 6, 3);	break;
				case ID_5D_HARD:	startGame (5, 60, 4, 5, 4, 7, 3);	break;
				case ID_6D_EASY:	startGame (6, 4, 3, 3, 3, 3, 3, 3);		break;
				case ID_6D_NORMAL:	startGame (6, 10, 4, 3, 4, 3, 3, 3);	break;
				case ID_6D_HARD:	startGame (6, 20, 4, 4, 4, 3, 3, 3);	break;
				
				case ID_7D_NORMAL:	startGame (7, 5,  4, 4, 3, 6, 3, 3, 3);		break;
				case ID_7D_HARD:	startGame (7, 20, 4, 4, 3, 6, 3, 3, 3);		break;
				case ID_7D_HARDER:	startGame (7, 40, 4, 4, 3, 6, 3, 3, 3);		break;
				case ID_8D_HARD:	startGame (8, 5,  4, 3, 3, 3, 3, 3, 3, 3);	break;
				case ID_8D_HARDER:	startGame (8, 15, 4, 3, 3, 3, 3, 3, 3, 3);	break;
				case ID_8D_HARDEST: startGame (8, 50, 4, 3, 3, 3, 3, 3, 3, 3);	break;
				case ID_9D_HARDER:		startGame (9, 5,  3, 3, 3, 3, 3, 3, 2, 3, 2);	break;
				case ID_9D_HARDEST:		startGame (9, 12, 3, 3, 3, 3, 3, 3, 2, 3, 2);	break;
				case ID_9D_IMPOSSIBLE:	startGame (9, 25, 3, 3, 3, 3, 3, 3, 2, 3, 2);	break;			
				case ID_10D_HARDEST:			startGame (10, 5,  3,3,3,3,3,2,2,2,2,2); break;
				case ID_10D_IMPOSSIBLE:			startGame (10, 8, 3,3,3,3,3,2,2,2,2,2); break;
				case ID_10D_REALLYIMPOSSIBLE:	startGame (10, 12, 3,3,3,3,3,2,2,2,2,2); break;
				
				case ID_GAME_RESTART: restartGame (); break;
				
				case ID_GAME_EXIT: SendMessage (hwnd, WM_CLOSE, 0, 0); break;
				
				case ID_OPTIONS_HIGHLIGHTPOSSIBLEBOMBS:
					CheckMenuItem (hmenu, ID_OPTIONS_HIGHLIGHTPOSSIBLEBOMBS, 
													(highlight_possible_bombs = !highlight_possible_bombs) ? 
														MF_CHECKED : MF_UNCHECKED);
				break; 
				
				case ID_HELP_ABOUT:
					DialogBox (hinstance, TEXT ("ABOUT"), hWindow, AboutDlgProc); break;
			}			
		return 0;
		
		case WM_CLOSE:
			SendMessage (hwnd, WM_DESTROY, 0, 0);
		return 0;
		
		case WM_DESTROY:// End the game and exit the application		
			gameEnd();
			PostQuitMessage (0);
			return 0;
	}
	return DefWindowProc(hWindow, msg, wParam, lParam);
}


//-------------------------------------------------------------------------------

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, PSTR szCmdLine, int iCmdShow)
{
// window initialization routine
	MSG         msg;
	WNDCLASS    wndclass;
	wndclass.style          = CS_HREDRAW | CS_VREDRAW;
	wndclass.lpfnWndProc    = WndProc;
	wndclass.cbClsExtra     = 0;
	wndclass.cbWndExtra     = 0;
	wndclass.hInstance      = hInstance;
	hinstance = hInstance;
	wndclass.hIcon          = LoadIcon(hinstance, game_title);
	wndclass.hCursor        = LoadCursor(NULL, IDC_ARROW);
	wndclass.hbrBackground  = (HBRUSH)GetStockObject (GRAY_BRUSH);;
	wndclass.lpszMenuName   = game_title;
	wndclass.lpszClassName  = game_title;

	if (!RegisterClass(&wndclass))
		return 0;

	hwnd = CreateWindow(game_title, "Minefield 6D", 							
							WS_POPUPWINDOW | WS_CAPTION | WS_MINIMIZEBOX, 
							CW_USEDEFAULT, CW_USEDEFAULT, 
							screen_width + 2 * GetSystemMetrics (SM_CXFIXEDFRAME), 
							screen_height + GetSystemMetrics (SM_CYCAPTION) + GetSystemMetrics (SM_CYMENU) + 2 * GetSystemMetrics (SM_CYFIXEDFRAME), 
							NULL, NULL, hInstance, NULL);
	if (!hwnd)
		return 0;
	hmenu = GetMenu (hwnd);
	
	ShowWindow (hwnd, iCmdShow);
	UpdateWindow (hwnd);
	
	restartGame (); // recreate the default field and set new size of window
	
	while (GetMessage (&msg, NULL, 0, 0))
	{
		TranslateMessage(&msg);
		DispatchMessage(&msg);	
	}
	return (int)msg.wParam;
}