#pragma once
#pragma comment(lib, "winmm.lib")

#include <vector>
#include <windows.h>
#include <tchar.h>
#include <cassert>

using namespace std;

const char game_title[] = "Minefield";	// name of the game. Used for title caption, menu and icon name, wndclass name and messages

int	field_volume (800),		// total number of tiles on the whole multidimensional field		
		screen_width (200),	// width of the game screen, width of game window is this plus BORDER 
		screen_height (400),// height of the game screen, height of game window is this plus BORDER plus menu 
		dims (6),					// number of dimensions in the field
		bombs (10),					// total number of bombs on the field
		marks (0),					// number of marked tiles on the field
		not_opened_cells (800);		// number of tiles that are still not opened
const int TILE_SIZE = 16,			// size of tile (in pixels)
		SPACE01 = 6,				// border between 2d tables when they are drawn on the screen
		SPACE23 = 6,				// border between 4d tables (each consists of 2d tables)				
		BORDER = 10;			// border around the whole large table drawn on the screen
	// alternative values for Iron man mode with >6 dims and twice smaller tiles
const int TILE_SIZE_2 = 8,
		SPACE01_2 = 1,
		SPACE23_2 = 1,
		SPACE45_2 = 1,
		SPACE67_2 = 1,
		SPACE89_2 = 1,
		SPACE1011_2 = 1,
		BORDER_2 = 2;
const int MAX_DIMS = 10;		// maximal number of dimensions that the user can set for the field
#define BOMB -1					// this number in neighbours vector indicates that the tile is a BOMB

vector<int> neighbors;	// value BOMB shows bomb, other values show how many bomb are adjacent to the tile
vector<int> tile_state;	// if tile is closed, flagged, opened, or highlighted
vector<int> pixel_pos_x, pixel_pos_y;	// position of upper-left corner of each tile (in pixels)

vector<int> field_size (MAX_DIMS, 1);	// width, hieght, length, and other metrics of the field
vector<int> multiplier (MAX_DIMS, 1);	// stored multiplyers for faster calculation of array index
vector<int> size_in_pixels (MAX_DIMS);			// size (in pixels) of a table of given dimension (e.g. if tile_size is 16 and field_size[0] = 10, then size_in_pixels[0] is 160, if field_size[1] = 5 then size_in_pixels[1] = 80, if size_in_pixels [2] = 2, then size_int pixels[2] is 320 (!), if field_size[3] = 3 then size_in_pixels[3] = 240 etc). 
// This is needed for faster calculation of what tile was hit by mouse click

enum {CLOSED, FLAGGED, OPENED, HIGHLIGHTED, HIGHLIGHTED_AND_FLAGGED, EXPLOSION};	// states of tile

bool highlight_possible_bombs (true);	// if highlighting of bomb candidates should be used
#define CURSOR_NOT_ON_ANY_TILE -2	// constant indicates that cursor is not over any tile, but over border
vector<int> cursor_tile_coord (MAX_DIMS);	// coords of the tile over which uper currently has his cursor
int cursor_tile_index (0);					// index of the tile at which cursor currently point

HINSTANCE	hinstance;			// global instance of the application
HDC		hdc;					// handle to device context of the main game window where all stuff is drawn
HWND	hwnd;					// handle to the main game window
HMENU hmenu;					// handle to the main game menu
HBITMAP image_tiles;			// a large image file where all tile sprites are stored in a table
HBITMAP image_tiles_2;			// the same image file but twice smaller (for >6 dims)
vector<HBITMAP> loaded_images;	// a list of pointers to any loaded images during the program session. Needed to delete any image at program termination

//----------------------------------------------------------------------
inline HBITMAP loadImage (char* resource_name)	// loads Image file by the name of resource (not ID number, but a string name)
{	
	BITMAP bitmap;
	HBITMAP hBitmap;
	hBitmap = LoadBitmap (hinstance, TEXT (resource_name));
	GetObject (hBitmap, sizeof (BITMAP), &bitmap);
	loaded_images.push_back (hBitmap);	// remember that this image was loaded (will be freed when game exits)
	return hBitmap;
}

//------------------------------------------------------------------------
inline void repaint ()
{
	RECT r;
	r.right = 0;
	r.top = 0;
	r.left = screen_width;
	r.bottom = screen_height;
	InvalidateRect (hwnd, &r, FALSE);
}

//-----------------------------------------------------------------------
int message (TCHAR * szFormat, ...)	
{
	TCHAR szBuffer [256] ;
	va_list pArgList ;
	// The va_start macro (defined in STDARG.H) is usually equivalent to:
	// pArgList = (char *) &szFormat + sizeof (szFormat) ;
	va_start (pArgList, szFormat) ;
	// The last argument to wvsprintf points to the arguments
	_vsntprintf (szBuffer, sizeof (szBuffer) / sizeof (TCHAR),	szFormat, pArgList) ;
	// The va_end macro just zeroes out pArgList for no good reason
	va_end (pArgList) ;
	return MessageBox (NULL, szBuffer, game_title, MB_OK);
}

//-------------------------------------------------------------------------
inline int randomInt(const int& max)
{	
	if (max <= 1)	return 0;
	srand(GetTickCount() + rand());
	return (rand() % max);
}

//------------------------------------------------------------------------------
inline void drawImagePart (HBITMAP hBitmap, int x, int y, int sx, int sy, int w, int h)
{
	if (hBitmap != NULL)
	{
		HDC hdcMem = CreateCompatibleDC (hdc) ;
		SelectObject (hdcMem, hBitmap) ;
		BitBlt (hdc, x, y, w, h, hdcMem, sx, sy, SRCCOPY) ;
		DeleteDC (hdcMem) ;
	}
}

//-------------------------------------------------------------------------------
inline int toIndex (const vector<int>& x)  //convert coords to index
{
	assert (x.size() == MAX_DIMS);
	if (x[0] == CURSOR_NOT_ON_ANY_TILE)	// in case the coords contain invalid parameter, return that
		return CURSOR_NOT_ON_ANY_TILE;
	int i = 0; 
		for(int j = 0; j != MAX_DIMS; ++j)
			i += x [j] * multiplier [j];
	return i;
}

//-------------------------------------------------------------------------------
inline bool nextFieldElement (vector<int>& i)	// moves to next element of the field by updating all its coords.
{
	assert (i.size() == MAX_DIMS);
	++i[0];
	for (int j = 0; j != MAX_DIMS - 1; ++j)
		if (i[j] >= field_size[j])
		{
			i[j] = 0; 
			++i[j+1];
		}
	if (i[MAX_DIMS - 1] >= field_size[MAX_DIMS - 1])
		return false;	// end of field reached
	return true;	// end of field not reached
}

//-------------------------------------------------------------------------------
inline void fillWithZeros (int (&i)[MAX_DIMS])
{
	for (int j = 0; j != MAX_DIMS; j++) 
		i[j] = 0;	
}



//-------------------------------------------------------------------------------
inline bool victoryAchieved()
{
	return (not_opened_cells <= bombs);
}

//-------------------------------------------------------------------------------
inline void drawImage (HBITMAP hBitmap, const int &x, const int &y, const int &w, const int &h)
{
	drawImagePart (hBitmap, x, y, 0, 0, w, h);
}

